@@ -1,3 +1,4 @@
+import re
 from collections import defaultdict
 from functools import lru_cache
 from textwrap import dedent, indent
@@ -19,11 +20,32 @@ from urlid_graph.utils import random_name
 from . import formatting
 
 
+REGEXP_CAMEL_CASE = re.compile("([A-Z])")
+
+
 def get_urlid_database_uri():
     db = settings.DATABASES[urlid_graph_settings.DJANGO_DATABASE]
     return f"postgres://{db['USER']}:{db['PASSWORD']}@{db['HOST']}:{db['PORT']}/{db['NAME']}"
 
 
+def slug_from_model_name(name):
+    """
+    >>> slug_from_model_name('SomeDataset')
+    'some-dataset'
+    >>> slug_from_model_name('MyTestModel')
+    'my-test'
+    """
+
+    name = REGEXP_CAMEL_CASE.sub(r"-\1", name).lower()
+    if name[0] == "-":
+        name = name[1:]
+    if name[-1] == "-":
+        name = name[:-1]
+    if name.endswith("-model"):
+        name = name[:-6]
+    return name
+
+
 @lru_cache(maxsize=64)
 def get_template(template_code):
     return Template(template_code)
@@ -458,3 +480,43 @@ class ObjectModelMixin(models.Model):
 
     class Meta:
         abstract = True
+
+
+class DatasetModel(models.Model):
+    object_uuid = models.UUIDField(
+        verbose_name="Object UUID",
+        help_text="UUID of described object",
+        null=False,
+        blank=False,
+        db_index=True,
+    )
+
+    @classmethod
+    def fields(cls):
+        from django.db import connection
+        return [
+            {
+                "name": field.name,
+                "title": field.verbose_name,
+                "type": field.db_type(connection),
+                "description": field.help_text,
+            }
+            for field in cls._meta.fields if field.name != "id"
+        ]
+
+    def serialize(self):
+        return [getattr(self, field["name"]) for field in type(self).fields()]
+
+    @classmethod
+    def subclasses(cls):
+        data = {}
+        for klass in cls.__subclasses__():
+            if hasattr(klass._meta, "slug"):
+                slug = klass._meta.slug
+            else:
+                slug = slug_from_model_name(klass.__name__)
+            data[slug] = klass
+        return data
+
+    class Meta:
+        abstract = True
